@@ -3,4 +3,4 @@
  FULL OUTER JOIN {{ y_schema }}.{{ y_table }} y
            ON {% for col in join_cols %}x.{{ col }} <=> y.{{ col }}{% if not loop.last %} AND {% endif %}
            {% endfor -%}
-           WHERE {% if left %}x{% else %}y{% endif %}.{{ join_cols[0] }} IS NULL
\ No newline at end of file
+           WHERE {% if x %}y{% else %}x{% endif %}.{{ join_cols[0] }} IS NULL
\ No newline at end of file
